'use strict';

define(['app'], function(app) {
	app.controller("panelSettings",
		["$rootScope", "$scope", "$routeParams", "$q", "$uibModal", "$location", "$timeout", "api", "lang", "util", "confirm",
			function ($rootScope, $scope, $routeParams, $q, $uibModal, $location, $timeout, api, lang, util, confirm) {

				$scope.autocomplete = { accountText: '' };
				$scope.changed = false;
				
				$scope.currentSection = { _id: 'settings' };
				$scope.panelsections = [
					{ _id: 'settings', name: lang.t("Settings") },
					{ _id: 'databases', name: lang.t("Databases Auth") },
					{ _id: 'tokens', name: lang.t("API Access Tokens") },
				];

				$scope.changePanelSection = function (section) {

					if(!$scope.changed) {
						$scope.currentSection = section;
						return;
					}

					$scope.sectionChanged = true;

					confirm.open({
						message: lang.t("You didn't saved your changes"),
						confirm: function () {},
						cancel: function () { 
							$scope.currentSection = section;
							$scope.saveData = util.duplicateObject($scope.details);
							$scope.changed = false;
						},
						cancelLabel: lang.t("Leave Section"),
						confirmLabel: lang.t("Stay On Section"),
					});
				};

				$scope.$on('saveChanges', function(event, data) {
					if(data.success) $scope.changed = false;
				});
				
				$scope.$watch("saveData", function(){
					$scope.changed = util.isChanged($scope.saveData, $scope.details);
				}, true);
			}
		]
	);
	app.controller("panelSettings_Settings",
		["$rootScope", "$scope", "$routeParams", "$q", "$uibModal", "$location", "$timeout", "api", "lang", "alert", "popup",
			function ($rootScope, $scope, $routeParams, $q, $uibModal, $location, $timeout, api, lang, alert, popup) {

				$scope.$emit('noSaveButtonChange', false);

				$scope.clearAccount = function () {
					$scope.autocomplete.accountText = '';
					$scope.selectAccount({username: ''});
				};

				$scope.selectAccount = function(account) {
					if(account === undefined) return;
					$scope.saveData.options.default_db_user = account.username;
					$scope.autocomplete.accountText = account.username;
				};

				$scope.searchAccounts = function(query) {
					if(query) {
						var deferred = $q.defer();
						api.listAccounts({
							data: {
								login_only: 0,
								filter: query,
								sort: { username: 1 }
							},
							success: function(data) {
								var results = [];
								for(var i = 0; i < data.accounts.length; i++) results.push(data.accounts[i]);
								deferred.resolve( results );
							},
							failed: function (message) {
								alert.error(message);
							}
						});

						return deferred.promise;
					}

					return [];
				};

				$scope.manageAccounts = function() {
					popup.open({
						template: 'accountSelection',
						scope: $scope,
						resolve: {
							account: function() { return $scope.saveData.options.default_db_user; }
						}
					}).result.then(function(account) {
						$scope.selectAccount(account);
					}, function () {});
				};

				$scope.createAccount = function() {
					popup.open({
						templateViews: 'plugins/panel',
						template: 'createAccount',
						controller: 'panelSettings_CreateAccount',
						scope: $scope
					}).result.then(function(username) {
						$scope.selectAccount({ username: username });
					}, function () {});
				};

				$scope.$watch('saveData.options.default_db_user', function () {
					if($scope.saveData.options === undefined) return;
					$scope.selectAccount({ username: $scope.saveData.options.default_db_user });
				});
			}
		]
	);

	app.controller("panelSettings_Databases",
		["$rootScope", "$scope",
			function ($rootScope, $scope) {
				$scope.$emit('noSaveButtonChange', false);

			}
		]
	);

	app.controller("panelSettings_Tokens",
		["$rootScope", "$scope", "$routeParams", "$q", "$uibModal", "$location", "$timeout", "api", "lang", "alert", "meta", "popup", "confirm", "util",
			function ($rootScope, $scope, $routeParams, $q, $uibModal, $location, $timeout, api, lang, alert, meta, popup, confirm, util) {

				$scope.$emit('noSaveButtonChange', true);

				$scope.loading = false;
				$scope.saveing = false;
				$scope.tokens = [];

				$scope.loaders = {
					delete: false
				};

				meta = meta.new("access_tokens");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("created");
				if(!meta.getSortDirection()) meta.setSortDirection("desc");
				meta.setSortFields(["created"]);
				meta.setTotalItems($scope.tokens.length);

				$scope.onClickDelete = function(token) {

					if($scope.saveing) return;

					$scope.saveing = true;
					$scope.loaders.delete = true;

					confirm.open({
						message: lang.t("This access token will be permanently deleted!"),
						confirm: function () {
							api.panel('deleteToken', {
								data: { _id: token._id },
								success: function(data, message) {
									$scope.saveing = false;
									$scope.loaders.delete = false;
									$scope.fetch();
									alert.success(message);
								},
								failed: function (message) {
									$scope.saveing = false;
									$scope.loaders.delete = false;
									alert.error(message);
								}
							});
						},
						cancel: function () {
							$scope.saveing = false;
							$scope.loaders.delete = false;
						}
					});
				};

				$scope.copyToken = function(token) {
					util.copyToClipboard(token);
					alert.success(lang.t("Access token copied to clipboard"));
				};
				
				$scope.manageToken = function(token) {
					popup.open({
						size: 'xl',
						templateViews: 'plugins/panel/views',
						template: 'tokenManage',
						controller: 'panelSettings_ManageToken',
						scope: $scope,
						resolve: {
							token: function () { return token; }
						}
					}).result.then(function(token) {
						$scope.fetch();
					}, function () {
						$scope.fetch();
					});
				};

				$scope.fetch = function() {
					if($scope.loading) return;
					$scope.loading = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: {},
						filter: meta.getFilter()
					};

					if($scope.filter) apiParams.find.type = $scope.filter;
					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					api.panel('listTokens', {
						data: apiParams,
						success: function (data) {
							$scope.tokens = data.tokens;
							$scope.loading = false;
							meta.setTotalItems(data.total);
							meta.calculate(data.tokens);
						},
						failed: function (message) {
							alert.error(message);
						}
					});
				};

				$timeout($scope.fetch);

			}
		]
	);
	
	app.controller("panelSettings_ManageToken", 
		["$uibModalInstance", "$uibModal", "$scope", "$q", "$timeout", "api", "lang", "alert", "util", "popup", "token", 
		function ($uibModalInstance, $uibModal, $scope, $q, $timeout, api, lang, alert, util, popup, token) {
		
		$scope.tokenData = null;

		$scope.details = token ? token : {
			description: '',
			ip: '',
			username: '',
		};
		
		$scope.saveData = util.duplicateObject($scope.details);

		$scope.clearAccount = function () {
			$scope.autocomplete.accountText = '';
			$scope.selectAccount({username: ''});
		};

		$scope.selectAccount = function(account) {
			if(account === undefined) return;
			$scope.saveData.username = account.username;
			$scope.autocomplete.accountText = account.username;
		};

		$scope.searchAccounts = function(query) {
			if(query) {
				var deferred = $q.defer();
				api.listAccounts({
					data: {
						login_only: 1,
						filter: query,
						sort: { username: 1 }
					},
					success: function(data) {
						var results = [];
						for(var i = 0; i < data.accounts.length; i++) results.push(data.accounts[i]);
						deferred.resolve( results );
					},
					failed: function (message) {
						alert.error(message);
					}
				});

				return deferred.promise;
			}

			return [];
		};

		$scope.manageAccounts = function() {
			popup.open({
				template: 'accountSelection',
				scope: $scope,
				resolve: {
					account: function() { return $scope.saveData.username; }
				}
			}).result.then(function(account) {
				$scope.selectAccount(account);
			}, function () {});
		};

		$scope.$watch('saveData.username', function () {
			$scope.selectAccount({ username: $scope.saveData.username });
		});

		$scope.saveChanges = function () {

			var apiParams = util.saveParams($scope.saveData, $scope.details);
			apiParams.action = $scope.details._id ? 'modify' : 'create';

			api.panel('manageToken', {
				data: apiParams,
				success: function(data, message) {
					if($scope.details._id) $uibModalInstance.close(data);
					else $scope.tokenData = data;
					alert.success(message);
				},
				failed: function (message) {
					alert.error(message);
				}
			})
		};

		$scope.done = function () {
			$uibModalInstance.close($scope.tokenData);
		};

		$scope.cancel = function () {
			$uibModalInstance.dismiss(lang.t('cancel'));
		};

	}]);

	app.controller("panelSettings_CreateAccount", ["$uibModalInstance", "$uibModal", "$scope", "$timeout", "api", "lang", "alert", function ($uibModalInstance, $uibModal, $scope, $timeout, api, lang, alert) {

		$scope.saveData = { account: '' };

		$scope.ok = function () {

			api.panel('createAccount', { 
				data: { 
					account: $scope.saveData.account 
				}, 
				success: function(data, message) {
					$uibModalInstance.close($scope.saveData.account);
					alert.success(message);
				},
				failed: function (message) {
					alert.error(message);
				}
			})
		};

		$scope.cancel = function () {
			$uibModalInstance.dismiss(lang.t('cancel'));
		};

	}]);

});